#!/usr/bin/env python
# Copyright 2012 Rondeau Research, LLC
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import scipy
import matplotlib.pyplot as plt
import matplotlib.animation as animation

def squarewave(N, dc):
    y = scipy.zeros(N)
    nones = N*dc
    y[0:nones] = scipy.ones(nones)
    return y

def circshift(x, n):
    L = len(x)
    y = scipy.zeros(L)
    y[0:n] = x[-n:]
    y[n:]  = x[0:-n]
    return y
    
class show_convolution:
    def __init__(self, fs, N):

        self.N = N
        self.t = scipy.arange(0, N/fs - 1.0/fs, 1.0/fs)
        self.x = squarewave(N, 0.4)
        self.h = squarewave(N, 0.1)
        self.data = scipy.zeros(N)
        
        # Move x data to center of display
        self.x = circshift(self.x, scipy.floor(N/4))
        
        if 0:
            f2 = plt.figure(2, figsize=(16,12), facecolor='w')
            s2 = f2.add_subplot(1,1,1)
            s2.plot(self.t, self.x, 'b')
            s2.plot(self.t, self.h, 'r')
            s2.set_ylim(-2, 2)
            s2.set_xlim(0, max(self.t))

        f1 = plt.figure(1, figsize=(16,12), facecolor='w')
        s1 = f1.add_subplot(1,1,1)
        self.line = s1.plot([], [], 'b')
        self.line += s1.plot([], [], 'r')
        self.line += s1.plot([], [], 'k')
        s1.set_ylim(-2, 8)
        s1.set_xlim(0, max(self.t))
    
        ani = animation.FuncAnimation(f1, self.update_data, frames=N,
                                      init_func=self.init_data,
                                      blit=True, save_count=10)
        plt.show()

    def update_data(self, i):
        if i == 0:
            self.data = scipy.zeros(self.N)

        self.data[i] = scipy.dot(self.x, self.h)
       
        self.line[0].set_ydata(self.x)
        self.line[0].set_xdata(self.t)

        self.line[1].set_ydata(self.h)
        self.line[1].set_xdata(self.t)

        self.line[2].set_ydata(self.data)
        self.line[2].set_xdata(self.t)

        self.h = circshift(self.h, 1)
        return self.line

    def init_data(self):
        return self.line
    
def main():
    N = 50
    fs = 1e6

    c = show_convolution(fs, N)

if __name__ == "__main__":
    main()
