#!/usr/bin/env python
# Copyright 2012 Rondeau Research, LLC
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import scipy
import matplotlib.pyplot as plt
import matplotlib.animation as animation

from scipy import fftpack
from gnuradio.filter import firdes

def squarewave(N, dc):
    y = scipy.zeros(N)
    nones = N*dc
    y[0:nones] = scipy.ones(nones)
    return y

def sinewave(N, fc, fs, dc):
    y = scipy.zeros(N)
    nmax = int(scipy.ceil(N*dc))
    n = scipy.arange(0, nmax, 1)
    y[0:nmax] = scipy.sin(2*scipy.pi*(float(fc)/float(fs))*n)
    return y

def lpf(N, bw, tb, fs):
    y = scipy.zeros(N)
    t = firdes.low_pass_2(1, fs, bw, tb, 60)
    print "LPF num taps: ", len(t)
    y[0:len(t)] = t
    return y
    
def circshift(x, n):
    L = len(x)
    y = scipy.zeros(L)
    y[0:n] = x[-n:]
    y[n:]  = x[0:-n]
    return y
    
class show_convolution:
    def __init__(self, fs, N):

        self.N = N
        self.t = scipy.arange(0, N/fs, 1.0/fs)
        self.h = lpf(N, 100, 20, fs)
        self.data = scipy.zeros(N)

        x1 = sinewave(N, 50, fs, 0.15)
        x2 = sinewave(N, 200, fs, 0.15)
        x3 = sinewave(N, 400, fs, 0.15)
        
        # Move x data to center of display
        x1 = circshift(x1, scipy.floor(N/4))
        x2 = circshift(x2, scipy.floor(2*N/4))
        x3 = circshift(x3, scipy.floor(3*N/4))
        self.x = x1 + x2 + x3

        self.h = circshift(self.h, scipy.floor(3*N/4))
        
        if 1:
            h = circshift(self.h, scipy.floor(N/2))
            f2 = plt.figure(1, figsize=(12,12), facecolor='w')
            s21 = f2.add_subplot(2,1,1)
            s21.plot(self.t, h, 'r')
            s21.set_xlim(0, max(self.t))
            s21.set_title("Filter: BW = 100 Hz", fontsize=24, fontweight="bold")

            freq = scipy.linspace(-fs/2, fs/2, 10000)
            H = fftpack.fftshift(fftpack.fft(h, 10000))
            s22 = f2.add_subplot(2,1,2)
            s22.plot(freq, 20.0*scipy.log10(abs(H)))
            s22.set_title("FFT of Filter", fontsize=24, fontweight="bold")

        f1 = plt.figure(2, figsize=(12,12), facecolor='w')
        self.s1 = f1.add_subplot(2,1,1)
        self.line = self.s1.plot([], [], 'b')
        self.line += self.s1.plot([], [], 'r')
        self.s1.set_ylim(-1.1, 1.1)
        self.s1.set_xlim(0, max(self.t))
        self.s1.set_title("Signals (50, 200, 400 Hz) and (visually scaled) Filter",
                     fontsize=24, fontweight='bold')

        self.s2 = f1.add_subplot(2,1,2)
        self.line += self.s2.plot([], [], 'k')
        self.s2.set_ylim(-1.1, 1.1)
        self.s2.set_xlim(0, max(self.t))
        self.s2.set_title("Filtered Signals", fontsize=24, fontweight='bold')
    
        ani = animation.FuncAnimation(f1, self.update_data, frames=N,
                                      init_func=self.init_data,
                                      blit=True)
        plt.show()

    def update_data(self, i):
        #data = scipy.convolve(self.x, self.h, mode='same')
        self.data[i] = scipy.dot(self.x, self.h)

        if i == 0:
            self.data = scipy.zeros(self.N)

        self.line[0].set_ydata(self.x)
        self.line[0].set_xdata(self.t)

        self.line[1].set_ydata(5*self.h)
        self.line[1].set_xdata(self.t)

        self.line[2].set_ydata(self.data)
        self.line[2].set_xdata(self.t)

        self.h = circshift(self.h, 1)
        return self.line

    def init_data(self):
        return self.line
    
def main():
    N = 200
    fs = 1000.0

    c = show_convolution(fs, N)

if __name__ == "__main__":
    main()
