#!/usr/bin/env python
##################################################
# Gnuradio Python Flow Graph
# Title: Fm Channelizer
# Generated: Mon Aug 25 18:47:22 2014
##################################################

from PyQt4 import Qt
from PyQt4.QtCore import QObject, pyqtSlot
from gnuradio import analog
from gnuradio import audio
from gnuradio import blocks
from gnuradio import eng_notation
from gnuradio import gr
from gnuradio import qtgui
from gnuradio.eng_option import eng_option
from gnuradio.filter import firdes
from gnuradio.filter import pfb
from optparse import OptionParser
import PyQt4.Qwt5 as Qwt
import osmosdr
import sip
import sys

from distutils.version import StrictVersion
class fm_channelizer(gr.top_block, Qt.QWidget):

    def __init__(self):
        gr.top_block.__init__(self, "Fm Channelizer")
        Qt.QWidget.__init__(self)
        self.setWindowTitle("Fm Channelizer")
        try:
             self.setWindowIcon(Qt.QIcon.fromTheme('gnuradio-grc'))
        except:
             pass
        self.top_scroll_layout = Qt.QVBoxLayout()
        self.setLayout(self.top_scroll_layout)
        self.top_scroll = Qt.QScrollArea()
        self.top_scroll.setFrameStyle(Qt.QFrame.NoFrame)
        self.top_scroll_layout.addWidget(self.top_scroll)
        self.top_scroll.setWidgetResizable(True)
        self.top_widget = Qt.QWidget()
        self.top_scroll.setWidget(self.top_widget)
        self.top_layout = Qt.QVBoxLayout(self.top_widget)
        self.top_grid_layout = Qt.QGridLayout()
        self.top_layout.addLayout(self.top_grid_layout)

        self.settings = Qt.QSettings("GNU Radio", "fm_channelizer")
        self.restoreGeometry(self.settings.value("geometry").toByteArray())


        ##################################################
        # Variables
        ##################################################
        self.samp_rate = samp_rate = 20e6
        self.ch_tb = ch_tb = 100e3
        self.ch_bw = ch_bw = 125e3
        self.center_freq = center_freq = 97.9e6
        self.atten = atten = 60
        self.volume = volume = 0.1
        self.rfgain = rfgain = 14
        self.pfb_taps = pfb_taps = firdes.low_pass_2(1, samp_rate, ch_bw, ch_tb, atten, firdes.WIN_HANN)
        self.ifgain = ifgain = 24
        self.channels = channels = 100
        self.channel = channel = 0
        self.ch_rate = ch_rate = 200e3
        self.cfreq = cfreq = center_freq
        self.bbgain = bbgain = 20
        self.audio_rate = audio_rate = 44.1e3
        self.address = address = ""

        ##################################################
        # Blocks
        ##################################################
        self._volume_layout = Qt.QVBoxLayout()
        self._volume_tool_bar = Qt.QToolBar(self)
        self._volume_layout.addWidget(self._volume_tool_bar)
        self._volume_tool_bar.addWidget(Qt.QLabel("Volume"+": "))
        class qwt_counter_pyslot(Qwt.QwtCounter):
            def __init__(self, parent=None):
                Qwt.QwtCounter.__init__(self, parent)
            @pyqtSlot('double')
            def setValue(self, value):
                super(Qwt.QwtCounter, self).setValue(value)
        self._volume_counter = qwt_counter_pyslot()
        self._volume_counter.setRange(0, 10, 0.1)
        self._volume_counter.setNumButtons(2)
        self._volume_counter.setValue(self.volume)
        self._volume_tool_bar.addWidget(self._volume_counter)
        self._volume_counter.valueChanged.connect(self.set_volume)
        self._volume_slider = Qwt.QwtSlider(None, Qt.Qt.Horizontal, Qwt.QwtSlider.BottomScale, Qwt.QwtSlider.BgSlot)
        self._volume_slider.setRange(0, 10, 0.1)
        self._volume_slider.setValue(self.volume)
        self._volume_slider.setMinimumWidth(200)
        self._volume_slider.valueChanged.connect(self.set_volume)
        self._volume_layout.addWidget(self._volume_slider)
        self.top_grid_layout.addLayout(self._volume_layout, 3,0,1,2)
        self._rfgain_layout = Qt.QVBoxLayout()
        self._rfgain_tool_bar = Qt.QToolBar(self)
        self._rfgain_layout.addWidget(self._rfgain_tool_bar)
        self._rfgain_tool_bar.addWidget(Qt.QLabel("RF Gain"+": "))
        class qwt_counter_pyslot(Qwt.QwtCounter):
            def __init__(self, parent=None):
                Qwt.QwtCounter.__init__(self, parent)
            @pyqtSlot('double')
            def setValue(self, value):
                super(Qwt.QwtCounter, self).setValue(value)
        self._rfgain_counter = qwt_counter_pyslot()
        self._rfgain_counter.setRange(0, 14, 14)
        self._rfgain_counter.setNumButtons(2)
        self._rfgain_counter.setValue(self.rfgain)
        self._rfgain_tool_bar.addWidget(self._rfgain_counter)
        self._rfgain_counter.valueChanged.connect(self.set_rfgain)
        self._rfgain_slider = Qwt.QwtSlider(None, Qt.Qt.Horizontal, Qwt.QwtSlider.BottomScale, Qwt.QwtSlider.BgSlot)
        self._rfgain_slider.setRange(0, 14, 14)
        self._rfgain_slider.setValue(self.rfgain)
        self._rfgain_slider.setMinimumWidth(200)
        self._rfgain_slider.valueChanged.connect(self.set_rfgain)
        self._rfgain_layout.addWidget(self._rfgain_slider)
        self.top_grid_layout.addLayout(self._rfgain_layout, 3,1,1,1)
        self._ifgain_layout = Qt.QVBoxLayout()
        self._ifgain_tool_bar = Qt.QToolBar(self)
        self._ifgain_layout.addWidget(self._ifgain_tool_bar)
        self._ifgain_tool_bar.addWidget(Qt.QLabel("IF Gain"+": "))
        class qwt_counter_pyslot(Qwt.QwtCounter):
            def __init__(self, parent=None):
                Qwt.QwtCounter.__init__(self, parent)
            @pyqtSlot('double')
            def setValue(self, value):
                super(Qwt.QwtCounter, self).setValue(value)
        self._ifgain_counter = qwt_counter_pyslot()
        self._ifgain_counter.setRange(0, 40, 8)
        self._ifgain_counter.setNumButtons(2)
        self._ifgain_counter.setValue(self.ifgain)
        self._ifgain_tool_bar.addWidget(self._ifgain_counter)
        self._ifgain_counter.valueChanged.connect(self.set_ifgain)
        self._ifgain_slider = Qwt.QwtSlider(None, Qt.Qt.Horizontal, Qwt.QwtSlider.BottomScale, Qwt.QwtSlider.BgSlot)
        self._ifgain_slider.setRange(0, 40, 8)
        self._ifgain_slider.setValue(self.ifgain)
        self._ifgain_slider.setMinimumWidth(200)
        self._ifgain_slider.valueChanged.connect(self.set_ifgain)
        self._ifgain_layout.addWidget(self._ifgain_slider)
        self.top_grid_layout.addLayout(self._ifgain_layout, 4,0,1,1)
        self._channel_layout = Qt.QVBoxLayout()
        self._channel_tool_bar = Qt.QToolBar(self)
        self._channel_layout.addWidget(self._channel_tool_bar)
        self._channel_tool_bar.addWidget(Qt.QLabel("Channel"+": "))
        class qwt_counter_pyslot(Qwt.QwtCounter):
            def __init__(self, parent=None):
                Qwt.QwtCounter.__init__(self, parent)
            @pyqtSlot('double')
            def setValue(self, value):
                super(Qwt.QwtCounter, self).setValue(value)
        self._channel_counter = qwt_counter_pyslot()
        self._channel_counter.setRange(0, channels, 1)
        self._channel_counter.setNumButtons(2)
        self._channel_counter.setValue(self.channel)
        self._channel_tool_bar.addWidget(self._channel_counter)
        self._channel_counter.valueChanged.connect(self.set_channel)
        self._channel_slider = Qwt.QwtSlider(None, Qt.Qt.Horizontal, Qwt.QwtSlider.BottomScale, Qwt.QwtSlider.BgSlot)
        self._channel_slider.setRange(0, channels, 1)
        self._channel_slider.setValue(self.channel)
        self._channel_slider.setMinimumWidth(200)
        self._channel_slider.valueChanged.connect(self.set_channel)
        self._channel_layout.addWidget(self._channel_slider)
        self.top_grid_layout.addLayout(self._channel_layout, 2,0,1,1)
        self._cfreq_layout = Qt.QVBoxLayout()
        self._cfreq_tool_bar = Qt.QToolBar(self)
        self._cfreq_layout.addWidget(self._cfreq_tool_bar)
        self._cfreq_tool_bar.addWidget(Qt.QLabel("Carrier Freq"+": "))
        class qwt_counter_pyslot(Qwt.QwtCounter):
            def __init__(self, parent=None):
                Qwt.QwtCounter.__init__(self, parent)
            @pyqtSlot('double')
            def setValue(self, value):
                super(Qwt.QwtCounter, self).setValue(value)
        self._cfreq_counter = qwt_counter_pyslot()
        self._cfreq_counter.setRange(87.9e6, 107.9e6, 200e3)
        self._cfreq_counter.setNumButtons(2)
        self._cfreq_counter.setValue(self.cfreq)
        self._cfreq_tool_bar.addWidget(self._cfreq_counter)
        self._cfreq_counter.valueChanged.connect(self.set_cfreq)
        self._cfreq_slider = Qwt.QwtSlider(None, Qt.Qt.Horizontal, Qwt.QwtSlider.BottomScale, Qwt.QwtSlider.BgSlot)
        self._cfreq_slider.setRange(87.9e6, 107.9e6, 200e3)
        self._cfreq_slider.setValue(self.cfreq)
        self._cfreq_slider.setMinimumWidth(200)
        self._cfreq_slider.valueChanged.connect(self.set_cfreq)
        self._cfreq_layout.addWidget(self._cfreq_slider)
        self.top_grid_layout.addLayout(self._cfreq_layout, 3,0,1,1)
        self._bbgain_layout = Qt.QVBoxLayout()
        self._bbgain_tool_bar = Qt.QToolBar(self)
        self._bbgain_layout.addWidget(self._bbgain_tool_bar)
        self._bbgain_tool_bar.addWidget(Qt.QLabel("BB Gain"+": "))
        class qwt_counter_pyslot(Qwt.QwtCounter):
            def __init__(self, parent=None):
                Qwt.QwtCounter.__init__(self, parent)
            @pyqtSlot('double')
            def setValue(self, value):
                super(Qwt.QwtCounter, self).setValue(value)
        self._bbgain_counter = qwt_counter_pyslot()
        self._bbgain_counter.setRange(0, 62, 2)
        self._bbgain_counter.setNumButtons(2)
        self._bbgain_counter.setValue(self.bbgain)
        self._bbgain_tool_bar.addWidget(self._bbgain_counter)
        self._bbgain_counter.valueChanged.connect(self.set_bbgain)
        self._bbgain_slider = Qwt.QwtSlider(None, Qt.Qt.Horizontal, Qwt.QwtSlider.BottomScale, Qwt.QwtSlider.BgSlot)
        self._bbgain_slider.setRange(0, 62, 2)
        self._bbgain_slider.setValue(self.bbgain)
        self._bbgain_slider.setMinimumWidth(200)
        self._bbgain_slider.valueChanged.connect(self.set_bbgain)
        self._bbgain_layout.addWidget(self._bbgain_slider)
        self.top_grid_layout.addLayout(self._bbgain_layout, 4,1,1,1)
        self.qtgui_time_sink_x_0 = qtgui.time_sink_c(
        	1024, #size
        	samp_rate/channels, #samp_rate
        	"", #name
        	1 #number of inputs
        )
        self.qtgui_time_sink_x_0.set_update_time(0.10)
        self.qtgui_time_sink_x_0.set_y_axis(-1, 1)
        
        self.qtgui_time_sink_x_0.set_y_label("Amplitude", "")
        
        self.qtgui_time_sink_x_0.enable_tags(-1, True)
        self.qtgui_time_sink_x_0.set_trigger_mode(qtgui.TRIG_MODE_FREE, qtgui.TRIG_SLOPE_POS, 0.0, 0, 0, "")
        self.qtgui_time_sink_x_0.enable_autoscale(False)
        self.qtgui_time_sink_x_0.enable_grid(False)
        
        labels = ["", "", "", "", "",
                  "", "", "", "", ""]
        widths = [1, 1, 1, 1, 1,
                  1, 1, 1, 1, 1]
        colors = ["blue", "red", "green", "black", "cyan",
                  "magenta", "yellow", "dark red", "dark green", "blue"]
        styles = [1, 1, 1, 1, 1,
                  1, 1, 1, 1, 1]
        markers = [-1, -1, -1, -1, -1,
                   -1, -1, -1, -1, -1]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
                  1.0, 1.0, 1.0, 1.0, 1.0]
        
        for i in xrange(2*1):
            if len(labels[i]) == 0:
                if(i % 2 == 0):
                    self.qtgui_time_sink_x_0.set_line_label(i, "Re{{Data {0}}}".format(i/2))
                else:
                    self.qtgui_time_sink_x_0.set_line_label(i, "Im{{Data {0}}}".format(i/2))
            else:
                self.qtgui_time_sink_x_0.set_line_label(i, labels[i])
            self.qtgui_time_sink_x_0.set_line_width(i, widths[i])
            self.qtgui_time_sink_x_0.set_line_color(i, colors[i])
            self.qtgui_time_sink_x_0.set_line_style(i, styles[i])
            self.qtgui_time_sink_x_0.set_line_marker(i, markers[i])
            self.qtgui_time_sink_x_0.set_line_alpha(i, alphas[i])
        
        self._qtgui_time_sink_x_0_win = sip.wrapinstance(self.qtgui_time_sink_x_0.pyqwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_time_sink_x_0_win, 1,1,1,1)
        self.qtgui_freq_sink_x_0_0 = qtgui.freq_sink_c(
        	1024, #size
        	firdes.WIN_BLACKMAN_hARRIS, #wintype
        	0, #fc
        	samp_rate/channels, #bw
        	"", #name
        	1 #number of inputs
        )
        self.qtgui_freq_sink_x_0_0.set_update_time(0.10)
        self.qtgui_freq_sink_x_0_0.set_y_axis(-140, 10)
        self.qtgui_freq_sink_x_0_0.enable_autoscale(False)
        self.qtgui_freq_sink_x_0_0.enable_grid(False)
        self.qtgui_freq_sink_x_0_0.set_fft_average(0.1)
        
        labels = ["", "", "", "", "",
                  "", "", "", "", ""]
        widths = [1, 1, 1, 1, 1,
                  1, 1, 1, 1, 1]
        colors = ["blue", "red", "green", "black", "cyan",
                  "magenta", "yellow", "dark red", "dark green", "dark blue"]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
                  1.0, 1.0, 1.0, 1.0, 1.0]
        for i in xrange(1):
            if len(labels[i]) == 0:
                self.qtgui_freq_sink_x_0_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_freq_sink_x_0_0.set_line_label(i, labels[i])
            self.qtgui_freq_sink_x_0_0.set_line_width(i, widths[i])
            self.qtgui_freq_sink_x_0_0.set_line_color(i, colors[i])
            self.qtgui_freq_sink_x_0_0.set_line_alpha(i, alphas[i])
        
        self._qtgui_freq_sink_x_0_0_win = sip.wrapinstance(self.qtgui_freq_sink_x_0_0.pyqwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_freq_sink_x_0_0_win, 1,0,1,1)
        self.qtgui_freq_sink_x_0 = qtgui.freq_sink_c(
        	4096, #size
        	firdes.WIN_FLATTOP, #wintype
        	cfreq, #fc
        	samp_rate, #bw
        	"", #name
        	1 #number of inputs
        )
        self.qtgui_freq_sink_x_0.set_update_time(0.10)
        self.qtgui_freq_sink_x_0.set_y_axis(-140, 10)
        self.qtgui_freq_sink_x_0.enable_autoscale(False)
        self.qtgui_freq_sink_x_0.enable_grid(False)
        self.qtgui_freq_sink_x_0.set_fft_average(0.1)
        
        labels = ["", "", "", "", "",
                  "", "", "", "", ""]
        widths = [1, 1, 1, 1, 1,
                  1, 1, 1, 1, 1]
        colors = ["blue", "red", "green", "black", "cyan",
                  "magenta", "yellow", "dark red", "dark green", "dark blue"]
        alphas = [1.0, 1.0, 1.0, 1.0, 1.0,
                  1.0, 1.0, 1.0, 1.0, 1.0]
        for i in xrange(1):
            if len(labels[i]) == 0:
                self.qtgui_freq_sink_x_0.set_line_label(i, "Data {0}".format(i))
            else:
                self.qtgui_freq_sink_x_0.set_line_label(i, labels[i])
            self.qtgui_freq_sink_x_0.set_line_width(i, widths[i])
            self.qtgui_freq_sink_x_0.set_line_color(i, colors[i])
            self.qtgui_freq_sink_x_0.set_line_alpha(i, alphas[i])
        
        self._qtgui_freq_sink_x_0_win = sip.wrapinstance(self.qtgui_freq_sink_x_0.pyqwidget(), Qt.QWidget)
        self.top_grid_layout.addWidget(self._qtgui_freq_sink_x_0_win, 0,0,1,2)
        self.pfb_decimator_ccf_0 = pfb.decimator_ccf(
        	  channels,
        	  (pfb_taps),
        	  channel,
        	  100,
                  True,
                  True)
        self.pfb_decimator_ccf_0.declare_sample_delay(0)
        	
        self.pfb_arb_resampler_xxx_0 = pfb.arb_resampler_fff(
        	  audio_rate/50000,
                  taps=None,
        	  flt_size=32)
        self.pfb_arb_resampler_xxx_0.declare_sample_delay(0)
        	
        self.osmosdr_source_0 = osmosdr.source( args="numchan=" + str(1) + " " + "hackrf" )
        self.osmosdr_source_0.set_sample_rate(samp_rate)
        self.osmosdr_source_0.set_center_freq(cfreq-0, 0)
        self.osmosdr_source_0.set_freq_corr(0, 0)
        self.osmosdr_source_0.set_dc_offset_mode(2, 0)
        self.osmosdr_source_0.set_iq_balance_mode(2, 0)
        self.osmosdr_source_0.set_gain_mode(False, 0)
        self.osmosdr_source_0.set_gain(rfgain, 0)
        self.osmosdr_source_0.set_if_gain(ifgain, 0)
        self.osmosdr_source_0.set_bb_gain(bbgain, 0)
        self.osmosdr_source_0.set_antenna("", 0)
        self.osmosdr_source_0.set_bandwidth(0, 0)
          
        self.blocks_multiply_const_vxx = blocks.multiply_const_vff((volume, ))
        self.audio_sink = audio.sink(int(audio_rate), "", True)
        self.analog_wfm_rcv = analog.wfm_rcv(
        	quad_rate=ch_rate,
        	audio_decimation=int(ch_rate/audio_rate),
        )
        self.analog_agc2_xx_0 = analog.agc2_cc(1e-1, 1e-2, 1.0, 1.0)
        self.analog_agc2_xx_0.set_max_gain(65536)

        ##################################################
        # Connections
        ##################################################
        self.connect((self.pfb_arb_resampler_xxx_0, 0), (self.blocks_multiply_const_vxx, 0))
        self.connect((self.blocks_multiply_const_vxx, 0), (self.audio_sink, 0))
        self.connect((self.analog_wfm_rcv, 0), (self.pfb_arb_resampler_xxx_0, 0))
        self.connect((self.osmosdr_source_0, 0), (self.analog_agc2_xx_0, 0))
        self.connect((self.analog_agc2_xx_0, 0), (self.pfb_decimator_ccf_0, 0))
        self.connect((self.analog_agc2_xx_0, 0), (self.qtgui_freq_sink_x_0, 0))
        self.connect((self.pfb_decimator_ccf_0, 0), (self.analog_wfm_rcv, 0))
        self.connect((self.pfb_decimator_ccf_0, 0), (self.qtgui_freq_sink_x_0_0, 0))
        self.connect((self.pfb_decimator_ccf_0, 0), (self.qtgui_time_sink_x_0, 0))


    def closeEvent(self, event):
        self.settings = Qt.QSettings("GNU Radio", "fm_channelizer")
        self.settings.setValue("geometry", self.saveGeometry())
        event.accept()

    def get_samp_rate(self):
        return self.samp_rate

    def set_samp_rate(self, samp_rate):
        self.samp_rate = samp_rate
        self.set_pfb_taps(firdes.low_pass_2(1, self.samp_rate, self.ch_bw, self.ch_tb, self.atten, firdes.WIN_HANN))
        self.qtgui_freq_sink_x_0_0.set_frequency_range(0, self.samp_rate/self.channels)
        self.qtgui_freq_sink_x_0.set_frequency_range(self.cfreq, self.samp_rate)
        self.qtgui_time_sink_x_0.set_samp_rate(self.samp_rate/self.channels)
        self.osmosdr_source_0.set_sample_rate(self.samp_rate)

    def get_ch_tb(self):
        return self.ch_tb

    def set_ch_tb(self, ch_tb):
        self.ch_tb = ch_tb
        self.set_pfb_taps(firdes.low_pass_2(1, self.samp_rate, self.ch_bw, self.ch_tb, self.atten, firdes.WIN_HANN))

    def get_ch_bw(self):
        return self.ch_bw

    def set_ch_bw(self, ch_bw):
        self.ch_bw = ch_bw
        self.set_pfb_taps(firdes.low_pass_2(1, self.samp_rate, self.ch_bw, self.ch_tb, self.atten, firdes.WIN_HANN))

    def get_center_freq(self):
        return self.center_freq

    def set_center_freq(self, center_freq):
        self.center_freq = center_freq
        self.set_cfreq(self.center_freq)

    def get_atten(self):
        return self.atten

    def set_atten(self, atten):
        self.atten = atten
        self.set_pfb_taps(firdes.low_pass_2(1, self.samp_rate, self.ch_bw, self.ch_tb, self.atten, firdes.WIN_HANN))

    def get_volume(self):
        return self.volume

    def set_volume(self, volume):
        self.volume = volume
        self.blocks_multiply_const_vxx.set_k((self.volume, ))
        Qt.QMetaObject.invokeMethod(self._volume_counter, "setValue", Qt.Q_ARG("double", self.volume))
        Qt.QMetaObject.invokeMethod(self._volume_slider, "setValue", Qt.Q_ARG("double", self.volume))

    def get_rfgain(self):
        return self.rfgain

    def set_rfgain(self, rfgain):
        self.rfgain = rfgain
        Qt.QMetaObject.invokeMethod(self._rfgain_counter, "setValue", Qt.Q_ARG("double", self.rfgain))
        Qt.QMetaObject.invokeMethod(self._rfgain_slider, "setValue", Qt.Q_ARG("double", self.rfgain))
        self.osmosdr_source_0.set_gain(self.rfgain, 0)

    def get_pfb_taps(self):
        return self.pfb_taps

    def set_pfb_taps(self, pfb_taps):
        self.pfb_taps = pfb_taps
        self.pfb_decimator_ccf_0.set_taps((self.pfb_taps))

    def get_ifgain(self):
        return self.ifgain

    def set_ifgain(self, ifgain):
        self.ifgain = ifgain
        Qt.QMetaObject.invokeMethod(self._ifgain_counter, "setValue", Qt.Q_ARG("double", self.ifgain))
        Qt.QMetaObject.invokeMethod(self._ifgain_slider, "setValue", Qt.Q_ARG("double", self.ifgain))
        self.osmosdr_source_0.set_if_gain(self.ifgain, 0)

    def get_channels(self):
        return self.channels

    def set_channels(self, channels):
        self.channels = channels
        self.qtgui_freq_sink_x_0_0.set_frequency_range(0, self.samp_rate/self.channels)
        self.qtgui_time_sink_x_0.set_samp_rate(self.samp_rate/self.channels)

    def get_channel(self):
        return self.channel

    def set_channel(self, channel):
        self.channel = channel
        Qt.QMetaObject.invokeMethod(self._channel_counter, "setValue", Qt.Q_ARG("double", self.channel))
        Qt.QMetaObject.invokeMethod(self._channel_slider, "setValue", Qt.Q_ARG("double", self.channel))
        self.pfb_decimator_ccf_0.set_channel(int(self.channel))

    def get_ch_rate(self):
        return self.ch_rate

    def set_ch_rate(self, ch_rate):
        self.ch_rate = ch_rate

    def get_cfreq(self):
        return self.cfreq

    def set_cfreq(self, cfreq):
        self.cfreq = cfreq
        Qt.QMetaObject.invokeMethod(self._cfreq_counter, "setValue", Qt.Q_ARG("double", self.cfreq))
        Qt.QMetaObject.invokeMethod(self._cfreq_slider, "setValue", Qt.Q_ARG("double", self.cfreq))
        self.qtgui_freq_sink_x_0.set_frequency_range(self.cfreq, self.samp_rate)
        self.osmosdr_source_0.set_center_freq(self.cfreq-0, 0)

    def get_bbgain(self):
        return self.bbgain

    def set_bbgain(self, bbgain):
        self.bbgain = bbgain
        Qt.QMetaObject.invokeMethod(self._bbgain_counter, "setValue", Qt.Q_ARG("double", self.bbgain))
        Qt.QMetaObject.invokeMethod(self._bbgain_slider, "setValue", Qt.Q_ARG("double", self.bbgain))
        self.osmosdr_source_0.set_bb_gain(self.bbgain, 0)

    def get_audio_rate(self):
        return self.audio_rate

    def set_audio_rate(self, audio_rate):
        self.audio_rate = audio_rate
        self.pfb_arb_resampler_xxx_0.set_rate(self.audio_rate/50000)

    def get_address(self):
        return self.address

    def set_address(self, address):
        self.address = address

if __name__ == '__main__':
    import ctypes
    import sys
    if sys.platform.startswith('linux'):
        try:
            x11 = ctypes.cdll.LoadLibrary('libX11.so')
            x11.XInitThreads()
        except:
            print "Warning: failed to XInitThreads()"
    parser = OptionParser(option_class=eng_option, usage="%prog: [options]")
    (options, args) = parser.parse_args()
    if(StrictVersion(Qt.qVersion()) >= StrictVersion("4.5.0")):
        Qt.QApplication.setGraphicsSystem(gr.prefs().get_string('qtgui','style','raster'))
    qapp = Qt.QApplication(sys.argv)
    tb = fm_channelizer()
    tb.start()
    tb.show()
    def quitting():
        tb.stop()
        tb.wait()
    qapp.connect(qapp, Qt.SIGNAL("aboutToQuit()"), quitting)
    qapp.exec_()
    tb = None #to clean up Qt widgets
